<?php

declare(strict_types=1);

namespace FiloBlu\Refilo\Helper;


use FiloBlu\Core\Model\Configuration;
use FiloBlu\Refilo\Model\Config\Seo\HrefLangConfig;
use FiloBlu\Refilo\Remote\Entity\UrlRewrite;
use Magento\Cms\Helper\Page as PageHelper;
use Magento\Framework\App\Config\ScopeConfigInterface;
use Magento\Framework\App\ResourceConnection;
use Magento\Framework\Exception\LocalizedException;
use Magento\Framework\Exception\NoSuchEntityException;
use Magento\Store\Api\Data\StoreInterface;
use Magento\Store\Model\ScopeInterface;
use Magento\Store\Model\Service\StoreConfigManager;
use Magento\Store\Model\Store;
use Magento\Store\Model\StoreManagerInterface;
use Zend_Db;

/**
 *
 */
class Url
{
    /** @var StoreConfigManager */
    protected $storeConfigManager;
    /** @var ResourceConnection */
    private $resourceConnection;
    /** @var array|null */
    private $storeUrls;
    /** @var array|null */
    private $storeBaseUrls;
    /** @var Data */
    private $helper;
    /** @var StoreManagerInterface */
    private $storeManager;
    /** @var HrefLangConfig */
    private $hrefLangConfig;
    /** @var array | null */
    private $seoBaseLocales;
    /** @var array | null */
    private $hrefLangConfigLookup;
    /** @var array|null */
    private $storeLocales;
    /** @var  array|null */
    private $storeCodes;
    /** @var ScopeConfigInterface */
    private $scopeConfig;
    /** @var array|null */
    private $homeUrls;
    /** @var  array|null */
    private $hreflangEnabled;
    /** @var Configuration */
    private $coreConfiguration;
    /** @var @var array|null */
    private $storeFilters;
    /** @var Configuration */
    private $defaultStoreByCode;

    /**
     * @param Configuration $coreConfiguration
     * @param Data $helper
     * @param HrefLangConfig $hrefLangConfig
     * @param ResourceConnection $resourceConnection
     * @param ScopeConfigInterface $scopeConfig
     * @param StoreConfigManager $storeConfigManager
     * @param StoreManagerInterface $storeManager
     */
    public function __construct(Configuration         $coreConfiguration,
                                Data                  $helper,
                                HrefLangConfig        $hrefLangConfig,
                                ResourceConnection    $resourceConnection,
                                ScopeConfigInterface  $scopeConfig,
                                StoreConfigManager    $storeConfigManager,
                                StoreManagerInterface $storeManager)
    {
        $this->resourceConnection = $resourceConnection;
        $this->helper = $helper;
        $this->storeManager = $storeManager;
        $this->hrefLangConfig = $hrefLangConfig;
        $this->storeConfigManager = $storeConfigManager;
        $this->scopeConfig = $scopeConfig;
        $this->coreConfiguration = $coreConfiguration;
    }

    /**
     * @param UrlRewrite $urlRewrite
     * @return array
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    public function getHreflang(UrlRewrite $urlRewrite): array
    {
        if (!$this->isHrefLangEnabled($urlRewrite)) {
            return [];
        }

        $storeFilters = $this->getStoreFilter($urlRewrite->getStore());

        if ($this->isHome($urlRewrite)) {

            $result = [];
            foreach ($this->storeManager->getStores() as $store) {
                $storeId = $store->getId();

                if (!in_array($storeId, $storeFilters)) {
                    continue;
                }

                $storeCode = $this->getStoreCodeById($store->getId());
                $configuration = $this->getHrefLangConfig($storeCode);
                $result[$storeCode] = [
                    'code' => $configuration['use_magento_lang_code'] ? $this->getStoreLocale($storeCode) : $this->getSeoBaseLocale($storeCode),
                    'url'  => $this->getStoreUrl($storeId)
                ];

                if ($urlRewrite->getStore() === $storeCode) {
                    $result['x-default'] = [
                        'code' => 'x-default',
                        'url'  => $this->getStoreUrl($storeId)
                    ];
                }
            }

            return $result;
        }

        $connection = $this->resourceConnection->getConnection();

        $select = $connection->select()
            ->from(
                ['u' => 'url_rewrite'],
                ['store_id' => 'u.store_id', 'request_path' => 'u.request_path']
            );
        $entityType = $urlRewrite->getType();

        switch ($entityType) {
            case 'cms-page':
                $select
                    ->joinInner(
                        ['cp' => 'cms_page'],
                        "cp.page_id = u.entity_id AND entity_type = '$entityType' AND cp.refilo_page_identifier = '{$urlRewrite->getPageIdentifier()}' AND cp.refilo_page_identifier IS NOT NULL",
                        ['refilo_page_identifier' => 'cp.refilo_page_identifier']
                    )
                    ->order('cp.refilo_page_identifier ASC');
                break;

            case 'category':
            case 'product':
                $select
                    ->where("u.entity_type = '$entityType' AND target_path = '{$urlRewrite->getTargetUrl()}'");
                break;
            case 'master_data':
                $select
                    ->where("u.entity_type = 'contenttype_content' AND target_path = '{$urlRewrite->getTargetUrl()}'");
                break;

            default:
                return [];
        }

        $result = [];
        $select->where('u.store_id IN (?)', $storeFilters);

        foreach ($connection->fetchAll($select, [], Zend_Db::FETCH_ASSOC) as $url) {
            $storeCode = $this->getStoreCodeById($url['store_id']);
            $storeId = $url['store_id'];
            $configuration = $this->getHrefLangConfig($storeCode);
            $result[$storeCode] = [
                'code' => $configuration['use_magento_lang_code'] ? $this->getStoreLocale($storeCode) : $this->getSeoBaseLocale($storeCode),
                'url'  => $this->getStoreUrl($storeId, $url['request_path'])
            ];
        }
        return $result;
    }

    /**
     * @param UrlRewrite $urlRewrite
     * @return bool
     */
    public function isHrefLangEnabled(UrlRewrite $urlRewrite): bool
    {
        if ($this->hreflangEnabled === null) {
            foreach ($this->storeManager->getStores() as $store) {
                $storeId = $store->getId();
                $enabled = $this->hrefLangConfig->isHrefLangEnabled($storeId);

                $this->hreflangEnabled[$store->getCode()] = [
                    'product'     => $enabled && $this->hrefLangConfig->isHrefLangEnabledForProduct($storeId),
                    'category'    => $enabled && $this->hrefLangConfig->isHrefLangEnabledForCategory($storeId),
                    'cms-page'    => $enabled && $this->hrefLangConfig->isHrefLangEnabledForCms($storeId),
                    'master_data' => $enabled
                ];
            }
        }

        $store = $urlRewrite->getStore();
        $type = $urlRewrite->getType();

        if (isset($this->hreflangEnabled[$store][$type])) {
            return $this->hreflangEnabled[$store][$type];
        }

        return false;
    }

    /**
     * @param UrlRewrite $urlRewrite
     * @return bool
     */
    public function isHome(UrlRewrite $urlRewrite): bool
    {
        if ($urlRewrite->getType() !== 'cms-page') {
            return false;
        }

        return in_array($urlRewrite->getPageIdentifier(), $this->getHomeUrls());
    }

    /**
     * @return array
     */
    public function getHomeUrls(): ?array
    {
        if ($this->homeUrls === null) {
            $this->homeUrls = [];
            foreach ($this->storeManager->getStores() as $store) {
                $this->homeUrls[$store->getCode()] = $this->scopeConfig->getValue(
                    PageHelper::XML_PATH_HOME_PAGE,
                    ScopeInterface::SCOPE_STORE,
                    $store
                );
            }
        }
        return $this->homeUrls;
    }

    /**
     * @param $storeId
     * @return mixed|string
     */
    public function getStoreCodeById($storeId)
    {
        if ($this->storeCodes === null) {
            $this->storeCodes = [];
            foreach ($this->storeManager->getStores() as $store) {
                $this->storeCodes[$store->getId()] = $store->getCode();
            }
        }

        return $this->storeCodes[$storeId];
    }

    /**
     * @param $storeCode
     * @return array|mixed
     */
    public function getHrefLangConfig($storeCode)
    {
        if ($this->hrefLangConfigLookup === null) {
            $this->hrefLangConfigLookup = [];
            foreach ($this->storeManager->getStores() as $store) {
                $this->hrefLangConfigLookup[$store->getCode()] = $this->hrefLangConfig->getConfig($store);
            }
        }

        return $this->hrefLangConfigLookup[$storeCode];
    }

    /**
     * @param $storeCode
     * @return mixed|string
     */
    public function getStoreLocale($storeCode)
    {
        if ($this->storeLocales === null) {
            $this->storeLocales = [];
            foreach ($this->storeConfigManager->getStoreConfigs() as $storeConfig) {
                $this->storeLocales[$storeConfig->getCode()] = $storeConfig->getLocale();
            }
        }

        return $this->storeLocales[$storeCode];
    }

    /**
     * @param $storeCode
     * @return mixed|string
     */
    public function getSeoBaseLocale($storeCode)
    {
        if ($this->seoBaseLocales === null) {
            $this->seoBaseLocales = [];
            foreach ($this->storeManager->getStores() as $store) {
                $this->seoBaseLocales[$store->getCode()] = $this->hrefLangConfig->getSeoLocaleCodeForStore($store);
            }
        }

        return $this->seoBaseLocales[$storeCode];
    }

    /**
     * @param $storeId
     * @return mixed
     */
    public function getStoreBaseUrl($storeId)
    {
        if ($this->storeBaseUrls === null) {
            $this->storeBaseUrls = [];
            foreach ($this->storeManager->getStores() as $store) {
                $this->storeBaseUrls[$store->getId()] = $this->helper->getStoreFrontBaseUrl($store);
            }
        }

        return $this->storeBaseUrls[$storeId];
    }

    /**
     * @param $storeCode
     * @return array|mixed
     * @throws LocalizedException
     */
    public function getStoreFilter($storeCode)
    {
        if ($this->storeFilters === null) {
            $this->storeFilters = [];
            foreach ($this->coreConfiguration->getWebsiteMapping() as $map) {
                $this->storeFilters = array_merge(
                    $this->storeFilters,
                    array_fill_keys($map->getStoreCodes(), $map->getStoreIds()));
            }
        }

        return $this->storeFilters[$storeCode];
    }

    /**
     * @param $storeCode
     * @return StoreInterface|Store
     * @throws LocalizedException
     */
    public function getDefaultStore($storeCode)
    {
        if(!$this->defaultStoreByCode) {

            $this->defaultStoreByCode = [];

            foreach( $this->coreConfiguration->getWebsiteMapping() as $map ) {

                $this->defaultStoreByCode = array_merge(
                    $this->defaultStoreByCode,
                    array_fill_keys($map->getStoreCodes(), $map->getDefaultStore())
                );
            }
        }

        return $this->defaultStoreByCode[$storeCode];
    }

    /**
     * @param $storeId
     * @param string|null $path
     * @return string
     * @throws NoSuchEntityException
     */
    public function getStoreUrl($storeId, string $path = null): string
    {
        if ($this->storeUrls === null) {
            $this->storeUrls = [];
            foreach ($this->storeManager->getStores() as $store) {
                $this->storeUrls[$store->getId()] = $this->helper->getFrontendUrl($store);
            }
        }

        return "{$this->storeUrls[$storeId]}$path";
    }

    /**
     * @param $storeCode
     * @return mixed
     */
    public function getHomeUrl($storeCode)
    {
        return $this->getHomeUrls()[$storeCode];
    }

    /**
     * @param UrlRewrite $urlRewrite
     * @return string|null
     * @throws NoSuchEntityException
     * @throws LocalizedException
     */
    public function getCanonicalUrl(UrlRewrite $urlRewrite)
    {
        $storeCode = $urlRewrite->getStore();

        if(!$this->hrefLangConfig->useCanonical()) {
            return null;
        }

        $storeId = $this->hrefLangConfig->getCanonicalStore($storeCode);

        if($storeId === 0) {
            $storeId = $this->getDefaultStore($storeCode)->getId();
        }

        if($this->isHome($urlRewrite)) {
            return $this->getStoreUrl($storeId);
        }

        $connection = $this->resourceConnection->getConnection();

        $identifier = $urlRewrite->getPageIdentifier();
        $targetUrl  = $urlRewrite->getTargetUrl();

        $select = $connection->select()->from(['u' => 'url_rewrite'], ['request_path' => 'u.request_path']);

        switch ($urlRewrite->getType())
        {
            case 'cms-page':
                $select
                    ->joinInner(
                        ['cp' => 'cms_page'],
                        "cp.page_id = u.entity_id AND entity_type = 'cms-page' AND cp.refilo_page_identifier = '$identifier' AND cp.refilo_page_identifier IS NOT NULL",
                        ['refilo_page_identifier' => 'cp.refilo_page_identifier']
                    )
                    ->order('cp.refilo_page_identifier ASC');
                break;
            case 'category':
                $select->where("u.entity_type = 'category' AND target_path = '$targetUrl'");
                break;
            case 'product':
                $select->where("u.entity_type = 'product' AND target_path = '$targetUrl'");
                break;
            case 'master_data':
                $select->where("u.entity_type = 'contenttype_content' AND target_path = '$targetUrl'");
                break;
            default:
                return null;
        }

        $select->where('u.store_id = ?', $storeId);
        $path = $connection->fetchOne($select);

        if(!$path) {
            return  null;
        }



        return $this->getStoreUrl($storeId, $path);
    }

}
